@@ -20,9 +20,10 @@ class AvoidPattern(Specification):
     location
       Location of the DNA segment on which to enforce the pattern e.g.
       ``Location(10, 45, 1)``. For patterns which are not palindromic,
-      the strand matters! use +1 for eliminating the pattern on the +1 strand
+      the strand matters! Use +1 for eliminating the pattern on the +1 strand
       only, -1 for eliminating the pattern on the -1 strand, and 0 for
-      eliminating the pattern on both strands.
+      eliminating the pattern on both strands. Default ``None`` enforces on
+      the whole sequence.
 
     strand
       Alternative way to set the strand, meant to be used in two cases only:
@@ -30,29 +31,34 @@ class AvoidPattern(Specification):
       the pattern should be avoided on both strands (otherwise, only the
       feature's strand will be considered).
       (2) if you want to create a specification without preset location, but
-      with a set strand: ``AvoidPattern('BsmBI_site', strand=1)``
-
+      with a set strand: ``AvoidPattern('BsmBI_site', strand=1)``.
+      The default 'from_location' uses the strand specified in ``location``,
+      or if that is ``None``, it sets both strands.
     """
 
     best_possible_score = 0
     priority = 1
     shorthand_name = "no"  # will appear as, for instance, @no(BsmBI_site)
 
-    def __init__(
-        self, pattern=None, location=None, strand="from_location", boost=1.0
-    ):
+    def __init__(self, pattern=None, location=None, strand="from_location", boost=1.0):
         """Initialize."""
         if isinstance(pattern, str):
             pattern = SequencePattern.from_string(pattern)
         self.pattern = pattern
         self.location = Location.from_data(location)
-        if strand != "from_location":
-            if strand == "both":
-                strand = 0
-            if strand not in [-1, 0, 1]:
-                raise ValueError("unknown strand: %s" % strand)
-            self.location.strand = strand
-        self.strand = strand
+
+        if strand == "from_location":
+            if self.location is None:
+                self.strand = 0
+            else:
+                self.strand = self.location.strand
+        elif strand == "both":
+            self.strand = 0
+        elif strand in [-1, 0, 1]:
+            self.strand = strand
+        else:
+            raise ValueError("unknown strand: %s" % strand)
+
         self.boost = boost
 
     def evaluate(self, problem):
@@ -72,7 +78,7 @@ class AvoidPattern(Specification):
             return "No %s" % self.pattern.name
         else:
             return "No %s" % self.pattern
-    
+
     def breach_label(self):
         if self.pattern.name is not None:
             return str(self.pattern.name)
@@ -80,7 +86,9 @@ class AvoidPattern(Specification):
             return str(self.pattern)
 
     def initialized_on_problem(self, problem, role="constraint"):
-        return self._copy_with_full_span_if_no_location(problem)
+        copy_of_constraint = self._copy_with_full_span_if_no_location(problem)
+        copy_of_constraint.location.strand = self.strand
+        return copy_of_constraint
 
     def localized(self, location, problem=None, with_righthand=True):
         """Localize the pattern to the given location. Taking into account the
