@@ -1,12 +1,10 @@
-"""Implement AvoidPattern"""
+"""Implement EnforcePatternOccurence"""
 
 from ..MutationSpace import MutationSpace
 from ..SequencePattern import SequencePattern, DnaNotationPattern
 from ..Location import Location
 from ..biotools import reverse_complement
-from ..DnaOptimizationProblem.DnaOptimizationProblem import (
-    DnaOptimizationProblem,
-)
+from ..DnaOptimizationProblem.DnaOptimizationProblem import DnaOptimizationProblem
 from ..DnaOptimizationProblem.NoSolutionError import NoSolutionError
 from ..Specification import Specification, SpecEvaluation
 
@@ -14,7 +12,7 @@ from .EnforceSequence import EnforceSequence
 
 
 class EnforcePatternOccurence(Specification):
-    """Enforce a number of occurences of the given pattern in the sequence.
+    """Enforce a number of occurrences of the given pattern in the sequence.
 
     Shorthand for annotations: "insert" (although this specification can be
     used to both insert new occurences of a pattern, or destroy supernumerary
@@ -27,24 +25,26 @@ class EnforcePatternOccurence(Specification):
       "BsmBI_site", etc.
 
     occurences
-      Desired number of occurences of the pattern.
+      Desired number of occurrences of the pattern.
 
     location
       Location of the DNA segment on which to enforce the pattern e.g.
-      ``Location(10, 45, 1)``
+      ``Location(10, 45, 1)``. Default ``None`` means the whole sequence.
 
     center
-      If true, new inserted patterns will prioritize locations at the center
+      If True, new inserted patterns will prioritize locations at the center
       of the specification's location. Else the insertion will happen at
       the beginning of the location.
 
     strand
       Alternative way to set the strand, meant to be used in two cases only:
-      (1) in a Genbank annotation by setting ``strand=both`` to indicate that
+      (1) in a Genbank annotation by setting ``strand='both'`` to indicate that
       the pattern could be on both strands (otherwise, only the
       feature's strand will be considered).
       (2) if you want to create a specification without preset location, but
-      with a set strand: ``EnforcePatternOccurence('BsmBI_site', strand=1)``
+      with a set strand: ``EnforcePatternOccurence('BsmBI_site', strand=1)``.
+      The default 'from_location' uses the strand specified in ``location``, or
+      if that is ``None``, it sets both strands.
     """
 
     best_possible_score = 0
@@ -65,19 +65,25 @@ class EnforcePatternOccurence(Specification):
             pattern = SequencePattern.from_string(pattern)
         self.pattern = pattern
         self.location = Location.from_data(location)
-        if strand != "from_location":
-            if strand == "both":
-                strand = 0
-            if strand not in [-1, 0, 1]:
-                raise ValueError("unknown strand: %s" % strand)
-            self.location.strand = strand
-        self.strand = strand
+        if strand == "from_location":
+            if self.location is None:
+                self.strand = 0
+            else:
+                self.strand = self.location.strand
+        elif strand == "both":
+            self.strand = 0
+        elif strand in [-1, 0, 1]:
+            self.strand = strand
+        else:
+            raise ValueError("unknown strand: %s" % strand)
         self.occurences = occurences
         self.center = center
         self.boost = boost
 
     def initialized_on_problem(self, problem, role=None):
-        return self._copy_with_full_span_if_no_location(problem)
+        copy_of_constraint = self._copy_with_full_span_if_no_location(problem)
+        copy_of_constraint.location.strand = self.strand
+        return copy_of_constraint
 
     def evaluate(self, problem):
         """Score the difference between expected and observed n_occurences."""
@@ -115,8 +121,8 @@ class EnforcePatternOccurence(Specification):
     def insert_pattern_in_problem(self, problem, reverse=False):
         """Insert the pattern in the problem's sequence by successive tries.
 
-        This heuristic is attempted to get the number of occurences in the
-        pattern from 0 to some number
+        This heuristic is attempted to get the number of occurrences in the
+        pattern from 0 to some number.
         """
         sequence_to_insert = self.pattern.sequence
         if reverse:
@@ -169,9 +175,7 @@ class EnforcePatternOccurence(Specification):
                 return
             n_matches = len(evaluation.data["matches"])
             if n_matches < self.occurences:
-                other_constraints = [
-                    c for c in problem.constraints if c is not self
-                ]
+                other_constraints = [c for c in problem.constraints if c is not self]
                 new_problem = problem
                 for i in range(self.occurences - n_matches):
                     new_occurence_cst = self.copy_with_changes(
